@@ -1,4 +1,3 @@
-#!/usr/bin/env perl
 #-*- Mode: perl; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 
 # replace.pl: Common in-line replacing stuff for the ximian-setup-tools backends.
@@ -77,7 +76,7 @@ sub run_entry
   }
   
   # OK, the given entry didn't have any array refs in it...
-  
+
   return -1 if (!&Utils::Parse::replace_hash_values ($ncp, $values_hash));
   push (@$ncp, $$values_hash{$key}) unless $key eq "_always_";
   $res = -1 if &$proc (@$ncp);
@@ -102,7 +101,7 @@ sub set_from_table
   my ($i, @cp, @files, $res);
 
   $$fn{"OLD_HASH"} = $old_hash;
-  
+
   foreach $i (@$table)
   {
     @cp = @$i;
@@ -112,13 +111,16 @@ sub set_from_table
     @files = &Utils::Parse::replace_files (shift (@cp), $fn);
     unshift @cp, @files if (scalar @files) > 0;
 
+    # treat empty values as undef
+    delete $$values_hash{$key} if ($$values_hash{$key} eq "");
+
     if ((exists $$values_hash{$key}) or ($key eq "_always_"))
     {
       $res = &run_entry ($values_hash, $key, $proc, \@cp, $$values_hash{$key});
     }
     elsif ((!exists $$values_hash{$key}) && (exists $$old_hash{$key}))
     {
-      # we need to remove all the instances of the known variables that doesn't exist in the XML
+      # we need to remove all the instances of the known variables that doesn't exist in the data structure
       $res = &run_entry ($values_hash, $key, $proc, \@cp, undef);
     }
   }
@@ -248,10 +250,10 @@ sub join_first_array
 # Escape $value in /bin/sh way, find/append key and set escaped value.
 sub set_sh
 {
-  my ($file, $key, $value) = @_;
+  my ($file, $key, $value, $unescaped) = @_;
   my $ret;
 
-  $value = &Utils::Parse::escape ($value);
+  $value = &Utils::Parse::escape ($value) unless $unescaped;
 
   &Utils::Report::enter ();
   &Utils::Report::do_report ("replace_sh", $key, $file);
@@ -478,6 +480,8 @@ sub set_chat
 
     while ($i ne "")
     {
+	 $i =~ s/^\s*//;
+
 	 # If it uses quotes. FIXME: Assuming they surround the whole string.
 	 if ($i =~ /^\'/)
 	 {
@@ -491,12 +495,13 @@ sub set_chat
 	   $found = $1;
 	   $quoted = 0;
 	 }
-	 
+
 	 # If it looks like what we're looking for,
 	 # substitute what is in parens with value.
 	 if ($found =~ /$re/i)
 	 {
 	   $substr = $1;
+	   $substr =~ s/\*/\\\*/g;
 	   $found =~ s/$substr/$value/i;
 
 	   if ($quoted == 1)
@@ -539,6 +544,7 @@ sub set_ini
   my ($file, $section, $var, $value) = @_;
   my ($buff, $i, $found_flag, $ret);
   my ($pre_space, $post_comment, $sec_save);
+  my ($escaped_section);
 
   &Utils::Report::enter ();
   &Utils::Report::do_report ("replace_ini", $var, $section, $file);
@@ -547,6 +553,7 @@ sub set_ini
 
   &Utils::File::join_buffer_lines ($buff);
   $found_flag = 0;
+  $escaped_section = Utils::Parse::escape ($section);
   
   foreach $i (@$buff)
   {
@@ -558,9 +565,9 @@ sub set_ini
     
     if ($i ne "")
     {
-      if ($i =~ /\[$section\]/i)
+      if ($i =~ /\[$escaped_section\]/i)
       {
-        $i =~ s/(\[$section\][ \t]*)//i;
+        $i =~ s/(\[$escaped_section\][ \t]*)//i;
         $sec_save = $1;
         $found_flag = 1;
       }
@@ -572,7 +579,7 @@ sub set_ini
           $i = "$var = $value\n$i" if ($value ne "");
           $found_flag = 2;
         }
-        
+
         if ($i =~ /^$var[ \t]*=/i)
         {
           if ($value ne "")
@@ -613,11 +620,13 @@ sub remove_ini_section
   my ($file, $section) = @_;
   my ($buff, $i, $found_flag, $ret);
   my ($pre_space, $post_comment, $sec_save);
+  my ($escaped_section);
 
   &Utils::Report::enter ();
   &Utils::Report::do_report ("replace_del_ini_sect", $section, $file);
 
   $buff = &Utils::File::load_buffer ($file);
+  $escaped_section = &Utils::Parse::escape ($section);
 
   &Utils::File::join_buffer_lines ($buff);
   $found_flag = 0;
@@ -632,9 +641,9 @@ sub remove_ini_section
     
     if ($i ne "")
     {
-      if ($i =~ /\[$section\]/i)
+      if ($i =~ /\[$escaped_section\]/i)
       {
-        $i =~ s/(\[$section\][ \t]*)//i;
+        $i =~ s/(\[$escaped_section\][ \t]*)//i;
         $found_flag = 1;
       }
       elsif ($found_flag && $i =~ /\[.+\]/i)
@@ -1182,7 +1191,7 @@ sub set_rcinet1conf
   $iface =~ s/eth//;
   $line = "$kw\[$iface\]";
 
-  $val = "\"$val\"" if ($val ne undef);
+  $val = "\"$val\"";
 
   return &split ($file, $line, "[ \t]*=[ \t]*", $val);
 }
@@ -1348,7 +1357,7 @@ sub set_ppp_options_connect
 
   &Utils::Report::enter ();
   &Utils::Report::do_report ("network_set_ppp_connect", $file);
-  $ret = &set_ppp_options_re ($file, "^connect", "connect \"/usr/sbin/chat -v -f $value\"");
+  $ret = &set_ppp_options_re ($file, "^connect", "connect \"/usr/sbin/chat -v -f /etc/chatscripts/$value\"");
   &Utils::Report::leave ();
   return $ret;
 }
